@@ -1,17 +1,3 @@
-# Copyright 2014 Thomas Amland <thomas.amland@gmail.com>
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 from __future__ import annotations
 
 import logging
@@ -19,15 +5,16 @@ import os
 import stat
 import time
 from queue import Empty
+from typing import TYPE_CHECKING
 
 import pytest
-
 from watchdog.events import (
     DirCreatedEvent,
     DirDeletedEvent,
     DirModifiedEvent,
     DirMovedEvent,
     FileClosedEvent,
+    FileClosedNoWriteEvent,
     FileCreatedEvent,
     FileDeletedEvent,
     FileModifiedEvent,
@@ -37,7 +24,9 @@ from watchdog.events import (
 from watchdog.utils import platform
 
 from .shell import mkdir, mkfile, mv, rm, touch
-from .utils import ExpectEvent, P, StartWatching, TestEventQueue
+
+if TYPE_CHECKING:
+    from .utils import ExpectEvent, P, StartWatching, TestEventQueue
 
 logging.basicConfig(level=logging.DEBUG)
 logger = logging.getLogger(__name__)
@@ -51,10 +40,7 @@ if platform.is_darwin():
 
 def rerun_filter(exc, *args):
     time.sleep(5)
-    if issubclass(exc[0], Empty) and platform.is_windows():
-        return True
-
-    return False
+    return bool(issubclass(exc[0], Empty) and platform.is_windows())
 
 
 @pytest.mark.flaky(max_runs=5, min_passes=1, rerun_filter=rerun_filter)
@@ -76,15 +62,11 @@ def test_create(p: P, event_queue: TestEventQueue, start_watching: StartWatching
         assert isinstance(event, FileClosedEvent)
 
 
-@pytest.mark.xfail(reason="known to be problematic")
-@pytest.mark.skipif(
-    not platform.is_linux(), reason="FileCloseEvent only supported in GNU/Linux"
-)
+@pytest.mark.skipif(not platform.is_linux(), reason="FileClosed*Event only supported in GNU/Linux")
 @pytest.mark.flaky(max_runs=5, min_passes=1, rerun_filter=rerun_filter)
-def test_close(p: P, event_queue: TestEventQueue, start_watching: StartWatching) -> None:
-    f_d = open(p("a"), "a")
-    start_watching()
-    f_d.close()
+def test_closed(p: P, event_queue: TestEventQueue, start_watching: StartWatching) -> None:
+    with open(p("a"), "a"):
+        start_watching()
 
     # After file creation/open in append mode
     event = event_queue.get(timeout=5)[0]
@@ -95,8 +77,16 @@ def test_close(p: P, event_queue: TestEventQueue, start_watching: StartWatching)
     assert os.path.normpath(event.src_path) == os.path.normpath(p(""))
     assert isinstance(event, DirModifiedEvent)
 
-    # After read-only, only IN_CLOSE_NOWRITE is emitted but not caught for now #747
-    open(p("a"), "r").close()
+    # After read-only, only IN_CLOSE_NOWRITE is emitted
+    open(p("a")).close()
+
+    event = event_queue.get(timeout=5)[0]
+    assert event.src_path == p("a")
+    assert isinstance(event, FileOpenedEvent)
+
+    event = event_queue.get(timeout=5)[0]
+    assert event.src_path == p("a")
+    assert isinstance(event, FileClosedNoWriteEvent)
 
     assert event_queue.empty()
 
@@ -236,7 +226,7 @@ def test_move_to(p: P, start_watching: StartWatching, expect_event: ExpectEvent)
     mkdir(p("dir1"))
     mkdir(p("dir2"))
     mkfile(p("dir1", "a"))
-    start_watching(p("dir2"))
+    start_watching(path=p("dir2"))
 
     mv(p("dir1", "a"), p("dir2", "b"))
 
@@ -246,20 +236,18 @@ def test_move_to(p: P, start_watching: StartWatching, expect_event: ExpectEvent)
         expect_event(DirModifiedEvent(p("dir2")))
 
 
-@pytest.mark.skipif(
-    not platform.is_linux(), reason="InotifyFullEmitter only supported in Linux"
-)
+@pytest.mark.skipif(not platform.is_linux(), reason="InotifyFullEmitter only supported in Linux")
 def test_move_to_full(p: P, event_queue: TestEventQueue, start_watching: StartWatching) -> None:
     mkdir(p("dir1"))
     mkdir(p("dir2"))
     mkfile(p("dir1", "a"))
-    start_watching(p("dir2"), use_full_emitter=True)
+    start_watching(path=p("dir2"), use_full_emitter=True)
     mv(p("dir1", "a"), p("dir2", "b"))
 
     event = event_queue.get(timeout=5)[0]
     assert isinstance(event, FileMovedEvent)
     assert event.dest_path == p("dir2", "b")
-    assert event.src_path is None  # Should equal None since the path was not watched
+    assert event.src_path == ""  # Should be blank since the path was not watched
 
 
 @pytest.mark.flaky(max_runs=5, min_passes=1, rerun_filter=rerun_filter)
@@ -267,7 +255,7 @@ def test_move_from(p: P, start_watching: StartWatching, expect_event: ExpectEven
     mkdir(p("dir1"))
     mkdir(p("dir2"))
     mkfile(p("dir1", "a"))
-    start_watching(p("dir1"))
+    start_watching(path=p("dir1"))
 
     mv(p("dir1", "a"), p("dir2", "b"))
 
@@ -277,20 +265,18 @@ def test_move_from(p: P, start_watching: StartWatching, expect_event: ExpectEven
         expect_event(DirModifiedEvent(p("dir1")))
 
 
-@pytest.mark.skipif(
-    not platform.is_linux(), reason="InotifyFullEmitter only supported in Linux"
-)
+@pytest.mark.skipif(not platform.is_linux(), reason="InotifyFullEmitter only supported in Linux")
 def test_move_from_full(p: P, event_queue: TestEventQueue, start_watching: StartWatching) -> None:
     mkdir(p("dir1"))
     mkdir(p("dir2"))
     mkfile(p("dir1", "a"))
-    start_watching(p("dir1"), use_full_emitter=True)
+    start_watching(path=p("dir1"), use_full_emitter=True)
     mv(p("dir1", "a"), p("dir2", "b"))
 
     event = event_queue.get(timeout=5)[0]
     assert isinstance(event, FileMovedEvent)
     assert event.src_path == p("dir1", "a")
-    assert event.dest_path is None  # Should equal None since path not watched
+    assert event.dest_path == ""  # Should be blank since path not watched
 
 
 @pytest.mark.flaky(max_runs=5, min_passes=1, rerun_filter=rerun_filter)
@@ -298,7 +284,7 @@ def test_separate_consecutive_moves(p: P, start_watching: StartWatching, expect_
     mkdir(p("dir1"))
     mkfile(p("dir1", "a"))
     mkfile(p("b"))
-    start_watching(p("dir1"))
+    start_watching(path=p("dir1"))
     mv(p("dir1", "a"), p("c"))
     mv(p("b"), p("dir1", "d"))
 
@@ -321,13 +307,11 @@ def test_separate_consecutive_moves(p: P, start_watching: StartWatching, expect_
 
 
 @pytest.mark.flaky(max_runs=5, min_passes=1, rerun_filter=rerun_filter)
-@pytest.mark.skipif(
-    platform.is_bsd(), reason="BSD create another set of events for this test"
-)
+@pytest.mark.skipif(platform.is_bsd(), reason="BSD create another set of events for this test")
 def test_delete_self(p: P, start_watching: StartWatching, expect_event: ExpectEvent) -> None:
     mkdir(p("dir1"))
-    emitter = start_watching(p("dir1"))
-    rm(p("dir1"), True)
+    emitter = start_watching(path=p("dir1"))
+    rm(p("dir1"), recursive=True)
     expect_event(DirDeletedEvent(p("dir1")))
     emitter.join(5)
     assert not emitter.is_alive()
@@ -342,10 +326,10 @@ def test_fast_subdirectory_creation_deletion(p: P, event_queue: TestEventQueue,
     sub_dir = p("dir1", "subdir1")
     times = 30
     mkdir(root_dir)
-    start_watching(root_dir)
+    start_watching(path=root_dir)
     for _ in range(times):
         mkdir(sub_dir)
-        rm(sub_dir, True)
+        rm(sub_dir, recursive=True)
         time.sleep(0.1)  # required for macOS emitter to catch up with us
     count = {DirCreatedEvent: 0, DirModifiedEvent: 0, DirDeletedEvent: 0}
     etype_for_dir = {
@@ -360,9 +344,7 @@ def test_fast_subdirectory_creation_deletion(p: P, event_queue: TestEventQueue,
         count[etype] += 1
         assert event.src_path == etype_for_dir[etype]
         assert count[DirCreatedEvent] >= count[DirDeletedEvent]
-        assert (
-            count[DirCreatedEvent] + count[DirDeletedEvent] >= count[DirModifiedEvent]
-        )
+        assert count[DirCreatedEvent] + count[DirDeletedEvent] >= count[DirModifiedEvent]
     assert count == {
         DirCreatedEvent: times,
         DirModifiedEvent: times * 2,
@@ -372,7 +354,7 @@ def test_fast_subdirectory_creation_deletion(p: P, event_queue: TestEventQueue,
 
 @pytest.mark.flaky(max_runs=5, min_passes=1, rerun_filter=rerun_filter)
 def test_passing_unicode_should_give_unicode(p: P, event_queue: TestEventQueue, start_watching: StartWatching) -> None:
-    start_watching(str(p()))
+    start_watching(path=str(p()))
     mkfile(p("a"))
     event = event_queue.get(timeout=5)[0]
     assert isinstance(event.src_path, str)
@@ -383,7 +365,7 @@ def test_passing_unicode_should_give_unicode(p: P, event_queue: TestEventQueue,
     reason="Windows ReadDirectoryChangesW supports only" " unicode for paths.",
 )
 def test_passing_bytes_should_give_bytes(p: P, event_queue: TestEventQueue, start_watching: StartWatching) -> None:
-    start_watching(p().encode())
+    start_watching(path=p().encode())
     mkfile(p("a"))
     event = event_queue.get(timeout=5)[0]
     assert isinstance(event.src_path, bytes)
@@ -391,7 +373,7 @@ def test_passing_bytes_should_give_bytes(p: P, event_queue: TestEventQueue, star
 
 @pytest.mark.flaky(max_runs=5, min_passes=1, rerun_filter=rerun_filter)
 def test_recursive_on(p: P, event_queue: TestEventQueue, start_watching: StartWatching) -> None:
-    mkdir(p("dir1", "dir2", "dir3"), True)
+    mkdir(p("dir1", "dir2", "dir3"), parents=True)
     start_watching()
     touch(p("dir1", "dir2", "dir3", "a"))
 
@@ -448,9 +430,7 @@ def test_recursive_off(
             event_queue.get(timeout=5)
 
         mkdir(p("dir3"))
-        expect_event(
-            DirModifiedEvent(p())
-        )  # the contents of the parent directory changed
+        expect_event(DirModifiedEvent(p()))  # the contents of the parent directory changed
 
         mv(p("dir1", "dir2", "somefile"), p("somefile"))
         expect_event(FileMovedEvent(p("dir1", "dir2", "somefile"), p("somefile")))
@@ -461,9 +441,7 @@ def test_recursive_off(
         expect_event(DirModifiedEvent(p()))
 
 
-@pytest.mark.skipif(
-    platform.is_windows(), reason="Windows create another set of events for this test"
-)
+@pytest.mark.flaky(max_runs=5, min_passes=1, rerun_filter=rerun_filter)
 def test_renaming_top_level_directory(
     p: P,
     event_queue: TestEventQueue,
@@ -474,7 +452,8 @@ def test_renaming_top_level_directory(
 
     mkdir(p("a"))
     expect_event(DirCreatedEvent(p("a")))
-    expect_event(DirModifiedEvent(p()))
+    if not platform.is_windows():
+        expect_event(DirModifiedEvent(p()))
 
     mkdir(p("a", "b"))
     expect_event(DirCreatedEvent(p("a", "b")))
@@ -482,10 +461,10 @@ def test_renaming_top_level_directory(
 
     mv(p("a"), p("a2"))
     expect_event(DirMovedEvent(p("a"), p("a2")))
-    expect_event(DirModifiedEvent(p()))
-    expect_event(DirModifiedEvent(p()))
-
-    expect_event(DirMovedEvent(p("a", "b"), p("a2", "b")))
+    if not platform.is_windows():
+        expect_event(DirModifiedEvent(p()))
+        expect_event(DirModifiedEvent(p()))
+    expect_event(DirMovedEvent(p("a", "b"), p("a2", "b"), is_synthetic=True))
 
     if platform.is_bsd():
         expect_event(DirModifiedEvent(p()))
@@ -500,19 +479,8 @@ def test_renaming_top_level_directory(
             break
 
     assert all(
-        [
-            isinstance(
-                e,
-                (
-                    FileCreatedEvent,
-                    FileMovedEvent,
-                    FileOpenedEvent,
-                    DirModifiedEvent,
-                    FileClosedEvent,
-                ),
-            )
-            for e in events
-        ]
+        isinstance(e, (FileCreatedEvent, FileMovedEvent, FileOpenedEvent, DirModifiedEvent, FileClosedEvent))
+        for e in events
     )
 
     for event in events:
@@ -525,74 +493,7 @@ def test_renaming_top_level_directory(
             assert event.src_path == p("a2", "b")
 
 
-@pytest.mark.flaky(max_runs=5, min_passes=1, rerun_filter=rerun_filter)
-@pytest.mark.skipif(
-    not platform.is_windows(),
-    reason="Non-Windows create another set of events for this test",
-)
-def test_renaming_top_level_directory_on_windows(
-    p: P,
-    event_queue: TestEventQueue,
-    start_watching: StartWatching,
-) -> None:
-    start_watching()
-
-    mkdir(p("a"))
-    event = event_queue.get(timeout=5)[0]
-    assert isinstance(event, DirCreatedEvent)
-    assert event.src_path == p("a")
-
-    mkdir(p("a", "b"))
-    event = event_queue.get(timeout=5)[0]
-    assert isinstance(event, DirCreatedEvent)
-    assert event.src_path == p("a", "b")
-
-    event = event_queue.get(timeout=5)[0]
-    assert isinstance(event, DirCreatedEvent)
-    assert event.src_path == p("a", "b")
-
-    event = event_queue.get(timeout=5)[0]
-    assert isinstance(event, DirModifiedEvent)
-    assert event.src_path == p("a")
-
-    mv(p("a"), p("a2"))
-    event = event_queue.get(timeout=5)[0]
-    assert isinstance(event, DirMovedEvent)
-    assert event.src_path == p("a", "b")
-
-    open(p("a2", "b", "c"), "a").close()
-
-    events = []
-    while True:
-        events.append(event_queue.get(timeout=5)[0])
-        if event_queue.empty():
-            break
-
-    assert all(
-        [
-            isinstance(
-                e, (FileCreatedEvent, FileMovedEvent, DirMovedEvent, DirModifiedEvent)
-            )
-            for e in events
-        ]
-    )
-
-    for event in events:
-        if isinstance(event, FileCreatedEvent):
-            assert event.src_path == p("a2", "b", "c")
-        elif isinstance(event, FileMovedEvent):
-            assert event.dest_path == p("a2", "b", "c")
-            assert event.src_path == p("a", "b", "c")
-        elif isinstance(event, DirMovedEvent):
-            assert event.dest_path == p("a2")
-            assert event.src_path == p("a")
-        elif isinstance(event, DirModifiedEvent):
-            assert event.src_path == p("a2", "b")
-
-
-@pytest.mark.skipif(
-    platform.is_windows(), reason="Windows create another set of events for this test"
-)
+@pytest.mark.skipif(platform.is_windows(), reason="Windows create another set of events for this test")
 def test_move_nested_subdirectories(
     p: P,
     event_queue: TestEventQueue,
@@ -608,8 +509,8 @@ def test_move_nested_subdirectories(
     expect_event(DirModifiedEvent(p("dir1")))
     expect_event(DirModifiedEvent(p()))
 
-    expect_event(DirMovedEvent(p("dir1", "dir2", "dir3"), p("dir2", "dir3")))
-    expect_event(FileMovedEvent(p("dir1", "dir2", "dir3", "a"), p("dir2", "dir3", "a")))
+    expect_event(DirMovedEvent(p("dir1", "dir2", "dir3"), p("dir2", "dir3"), is_synthetic=True))
+    expect_event(FileMovedEvent(p("dir1", "dir2", "dir3", "a"), p("dir2", "dir3", "a"), is_synthetic=True))
 
     if platform.is_bsd():
         event = event_queue.get(timeout=5)[0]
@@ -644,7 +545,7 @@ def test_move_nested_subdirectories_on_windows(
 ) -> None:
     mkdir(p("dir1/dir2/dir3"), parents=True)
     mkfile(p("dir1/dir2/dir3", "a"))
-    start_watching(p(""))
+    start_watching(path=p(""))
     mv(p("dir1/dir2"), p("dir2"))
 
     event = event_queue.get(timeout=5)[0]
@@ -671,7 +572,7 @@ def test_move_nested_subdirectories_on_windows(
         if event_queue.empty():
             break
 
-    assert all([isinstance(e, (FileModifiedEvent, DirModifiedEvent)) for e in events])
+    assert all(isinstance(e, (FileModifiedEvent, DirModifiedEvent)) for e in events)
 
     for event in events:
         if isinstance(event, FileModifiedEvent):
@@ -681,9 +582,7 @@ def test_move_nested_subdirectories_on_windows(
 
 
 @pytest.mark.flaky(max_runs=5, min_passes=1, rerun_filter=rerun_filter)
-@pytest.mark.skipif(
-    platform.is_bsd(), reason="BSD create another set of events for this test"
-)
+@pytest.mark.skipif(platform.is_bsd(), reason="BSD create another set of events for this test")
 def test_file_lifecyle(p: P, start_watching: StartWatching, expect_event: ExpectEvent) -> None:
     start_watching()
 
