@@ -1,41 +1,22 @@
-# Copyright 2011 Yesudeep Mangalapilly <yesudeep@gmail.com>
-# Copyright 2012 Google, Inc & contributors.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 from __future__ import annotations
 
 import os
 import os.path
-import sys
 from queue import Empty, Queue
 from time import sleep
 
 import pytest
-
 from watchdog.events import DirCreatedEvent, DirMovedEvent
 from watchdog.observers.api import ObservedWatch
+from watchdog.utils import platform
 
 from .shell import mkdir, mkdtemp, mv, rm
 
 # make pytest aware this is windows only
-if not sys.platform.startswith("win"):
+if not platform.is_windows():
     pytest.skip("Windows only.", allow_module_level=True)
 
-# make mypy aware this is windows only and provide a clear runtime error just in case
-assert sys.platform.startswith("win"), f"{__name__} requires Windows"
-
-from watchdog.observers.read_directory_changes import WindowsApiEmitter  # noqa: E402
+from watchdog.observers.read_directory_changes import WindowsApiEmitter
 
 SLEEP_TIME = 2
 
@@ -52,14 +33,14 @@ def p(*args):
     return os.path.join(temp_dir, *args)
 
 
-@pytest.fixture
+@pytest.fixture()
 def event_queue():
-    yield Queue()
+    return Queue()
 
 
-@pytest.fixture
+@pytest.fixture()
 def emitter(event_queue):
-    watch = ObservedWatch(temp_dir, True)
+    watch = ObservedWatch(temp_dir, recursive=True)
     em = WindowsApiEmitter(event_queue, watch, timeout=0.2)
     yield em
     em.stop()
@@ -121,13 +102,13 @@ def test_root_deleted(event_queue, emitter):
         File "watchdog\observers\read_directory_changes.py", line 76, in queue_events
             winapi_events = self._read_events()
         File "watchdog\observers\read_directory_changes.py", line 73, in _read_events
-            return read_events(self._handle, self.watch.path, self.watch.is_recursive)
+            return read_events(self._whandle, self.watch.path, recursive=self.watch.is_recursive)
         File "watchdog\observers\winapi.py", line 387, in read_events
-            buf, nbytes = read_directory_changes(handle, path, recursive)
+            buf, nbytes = read_directory_changes(handle, path, recursive=recursive)
         File "watchdog\observers\winapi.py", line 340, in read_directory_changes
             return _generate_observed_path_deleted_event()
         File "watchdog\observers\winapi.py", line 298, in _generate_observed_path_deleted_event
-            event = FILE_NOTIFY_INFORMATION(0, FILE_ACTION_DELETED_SELF, len(path), path.value)
+            event = FileNotifyInformation(0, FILE_ACTION_DELETED_SELF, len(path), path.value)
         TypeError: expected bytes, str found
     """
 
