@@ -1,37 +1,27 @@
-# Copyright (C) 2010 Yesudeep Mangalapilly <yesudeep@gmail.com>
-# Copyright 2020 Boris Staletic <boris.staletic@gmail.com>
-
 from __future__ import annotations
 
 import pytest
-
 from watchdog.utils.patterns import _match_path, filter_paths, match_any_paths
 
 
 @pytest.mark.parametrize(
-    "input, included_patterns, excluded_patterns, case_sensitive, expected",
+    ("raw_path", "included_patterns", "excluded_patterns", "case_sensitive", "expected"),
     [
-        ("/users/gorakhargosh/foobar.py", {"*.py"}, {"*.PY"}, True, True),
         ("/users/gorakhargosh/foobar.py", {"*.py"}, {"*.PY"}, True, True),
         ("/users/gorakhargosh/", {"*.py"}, {"*.txt"}, False, False),
         ("/users/gorakhargosh/foobar.py", {"*.py"}, {"*.PY"}, False, ValueError),
     ],
 )
-def test_match_path(
-    input, included_patterns, excluded_patterns, case_sensitive, expected
-):
-    if expected == ValueError:
+def test_match_path(raw_path, included_patterns, excluded_patterns, case_sensitive, expected):
+    if expected is ValueError:
         with pytest.raises(expected):
-            _match_path(input, included_patterns, excluded_patterns, case_sensitive)
+            _match_path(raw_path, included_patterns, excluded_patterns, case_sensitive=case_sensitive)
     else:
-        assert (
-            _match_path(input, included_patterns, excluded_patterns, case_sensitive)
-            is expected
-        )
+        assert _match_path(raw_path, included_patterns, excluded_patterns, case_sensitive=case_sensitive) is expected
 
 
 @pytest.mark.parametrize(
-    "included_patterns, excluded_patterns, case_sensitive, expected",
+    ("included_patterns", "excluded_patterns", "case_sensitive", "expected"),
     [
         (None, None, True, None),
         (None, None, False, None),
@@ -51,13 +41,18 @@ def test_filter_paths(included_patterns, excluded_patterns, case_sensitive, expe
         "/usr/local/bin/python",
     }
     actual = set(
-        filter_paths(pathnames, included_patterns, excluded_patterns, case_sensitive)
+        filter_paths(
+            pathnames,
+            included_patterns=included_patterns,
+            excluded_patterns=excluded_patterns,
+            case_sensitive=case_sensitive,
+        )
     )
     assert actual == expected if expected else pathnames
 
 
 @pytest.mark.parametrize(
-    "included_patterns, excluded_patterns, case_sensitive, expected",
+    ("included_patterns", "excluded_patterns", "case_sensitive", "expected"),
     [
         (None, None, True, True),
         (None, None, False, True),
@@ -66,9 +61,7 @@ def test_filter_paths(included_patterns, excluded_patterns, case_sensitive, expe
         (["*.txt"], None, True, False),
     ],
 )
-def test_match_any_paths(
-    included_patterns, excluded_patterns, case_sensitive, expected
-):
+def test_match_any_paths(included_patterns, excluded_patterns, case_sensitive, expected):
     pathnames = {
         "/users/gorakhargosh/foobar.py",
         "/var/cache/pdnsd.status",
@@ -76,6 +69,11 @@ def test_match_any_paths(
         "/usr/local/bin/python",
     }
     assert (
-        match_any_paths(pathnames, included_patterns, excluded_patterns, case_sensitive)
+        match_any_paths(
+            pathnames,
+            included_patterns=included_patterns,
+            excluded_patterns=excluded_patterns,
+            case_sensitive=case_sensitive,
+        )
         == expected
     )
